@    300    IN    SOA   {{soa_nameserver}} {{soa_mailbox}}  {{serial_number}}   43200  10800  1209600  300
;
;; this file is autogenerated by edgemanage ;;
;; based on edgemanage/template/zonetemplate.j2 ;;

{% for nameserver in nameservers -%}
@		IN	NS	{{nameserver}}
{% endfor -%}
;;;
;;;
;;;
;;;

{% for edge_ip in live_edge_ips -%}
@		IN	A	{{edge_ip}}
{% endfor -%}

{% for rotate_zone in rotate_zones -%}
{% for edge_ip in live_edge_ips -%}
{{rotate_zone}}		IN	A	{{edge_ip}}
{% endfor -%}
{% endfor -%}

;;; contents of {{domain}}.zone

{{zonefile}}
